@@ -10,24 +10,34 @@ from ..util import save_file, idle, get_project_root, single_ref
 from ..signals import SignalManager, Signal, connect_all, connect_external
 
 class Editor(SignalManager):
-    editor_closed = Signal()
-    request_to_open_file = Signal(str, object, return_type=object)
-    request_close = Signal()
-    settings_changed = Signal()
-    get_title = Signal(return_type=str)
-    get_file_position = Signal(return_type=int)
-    before_close = Signal()
-    file_loaded = Signal()
-    change_title = Signal(str)
-    request_transient_for = Signal(object)
-    file_saved = Signal()
-    push_escape_callback = Signal(object, object)
-    plugins_changed = Signal()
     add_spot_request = Signal()
+
+    before_close = Signal()
+    before_file_save = Signal(return_type=bool) # Handlers can return True to prevent file saving
+
+    change_title = Signal(str)
+    editor_closed = Signal()
+
+    file_loaded = Signal()
+    file_saved = Signal()
+
+    get_file_position = Signal(return_type=int)
+    get_project_larva = Signal(return_type=str)
+    get_title = Signal(return_type=str)
+    get_window_title = Signal(return_type=str)
+
+    plugins_changed = Signal()
+    push_escape_callback = Signal(object, object)
+
+    request_close = Signal()
+    request_to_open_file = Signal(str, object, object, return_type=object)
+    request_transient_for = Signal(object)
+
+    settings_changed = Signal()
     stack_add_request = Signal(object, object)
     stack_popup_request = Signal(object)
 
-    def __init__(self):
+    def __init__(self, snaked_conf):
         self.uri = None
         self.session = None
         self.saveable = True
@@ -54,9 +64,12 @@ class Editor(SignalManager):
 
         connect_all(self, buffer=self.buffer, view=self.view)
 
-    def update_title(self):
-        modified = '*' if self.buffer.get_modified() else ''
+        if snaked_conf['DISABLE_LEFT_CLICK']:
+            self.view.connect('button-press-event', self.on_button_press_event)
 
+        self.snaked_conf = snaked_conf
+
+    def update_title(self):
         if self.uri:
             title = self.get_title.emit()
 
@@ -65,7 +78,7 @@ class Editor(SignalManager):
         else:
             title = 'Unknown'
 
-        self.change_title.emit(modified + title)
+        self.change_title.emit(title)
 
     def load_file(self, filename, line=None):
         self.uri = os.path.abspath(filename)
@@ -84,9 +97,13 @@ class Editor(SignalManager):
                 try:
                     import chardet
                     result = chardet.detect(text)
-                    utext = text.decode(result['encoding'])
-                    self.encoding = result['encoding']
-                    idle(self.message, 'Automatically selected ' + self.encoding + 'encoding', 5000)
+                    if result['encoding']:
+                        utext = text.decode(result['encoding'])
+                        self.encoding = result['encoding']
+                        idle(self.message, 'Automatically selected ' + self.encoding + 'encoding', 5000)
+                    else:
+                        self.saveable = False
+                        utext = 'Is this a text file?'
                 except ImportError:
                     self.saveable = False
                     utext = str(e)
@@ -98,6 +115,15 @@ class Editor(SignalManager):
             self.buffer.set_modified(False)
 
             self.buffer.end_not_undoable_action()
+
+            if 'MODIFIED_FILES' in self.snaked_conf and self.uri in self.snaked_conf['MODIFIED_FILES']:
+                tmpfilename = self.snaked_conf['MODIFIED_FILES'][self.uri]
+                if os.path.exists(tmpfilename):
+                    self.buffer.begin_user_action()
+                    utext = open(tmpfilename).read().decode(self.encoding)
+                    self.buffer.set_text(utext)
+                    self.buffer.end_user_action()
+
             self.on_modified_changed_handler.unblock()
             self.view.window.thaw_updates()
 
@@ -122,6 +148,9 @@ class Editor(SignalManager):
             return
 
         if self.uri:
+            if self.before_file_save.emit():
+                return
+
             if self.prefs['remove-trailing-space']:
                 from snaked.core.processors import remove_trailing_spaces
                 remove_trailing_spaces(self.buffer)
@@ -137,18 +166,32 @@ class Editor(SignalManager):
 
     @property
     def project_root(self):
-        if self.uri:
-            root = get_project_root(self.uri)
+        """This is real file's project root
+
+        There config can be saved and so on.
+
+        """
+        return self.get_project_root()
+
+    def get_project_root(self, larva=False, force_select=False):
+        if not self.uri:
+            return None
+
+        root = get_project_root(self.uri)
+
+        if not root and force_select:
+            pass
+
+        if not root and larva:
+            root = self.get_project_larva.emit()
             if not root:
                 root = os.path.dirname(self.uri)
 
-            return root
+        return root
 
-        return None
-
-    def open_file(self, filename, line=None):
+    def open_file(self, filename, line=None, lang_id=None):
         """:rtype: Editor"""
-        return self.request_to_open_file.emit(filename, line)
+        return self.request_to_open_file.emit(filename, line, lang_id)
 
     @property
     def cursor(self):
@@ -166,18 +209,43 @@ class Editor(SignalManager):
         """
         return self.buffer.get_text(*self.buffer.get_bounds())
 
+    @property
+    def selection(self):
+        """Return buffer's selection or none as utf-8 encoded string
+
+        :rtype: str
+        """
+        if self.buffer.get_has_selection():
+            return self.buffer.get_text(*self.buffer.get_selection_bounds())
+        else:
+            None
+
+    @property
+    def uselection(self):
+        """Return buffer's selection or none as unicode string
+
+        :rtype: str
+        """
+        if self.buffer.get_has_selection():
+            return self.selection
+        else:
+            None
+
     @property
     def utext(self):
         """Return buffer's content as unicode string
 
         :rtype: unicode
         """
-        return unicode(self.buffer.get_text(*self.buffer.get_bounds()), 'utf-8')
+        return unicode(self.text, 'utf-8')
 
-    def goto_line(self, line):
+    def goto_line(self, line, minimal=False):
         iterator = self.buffer.get_iter_at_line(line - 1)
         self.buffer.place_cursor(iterator)
-        self.view.scroll_to_mark(self.buffer.get_insert(), 0.001, use_align=True, xalign=1.0)
+        if minimal:
+            self.view.scroll_mark_onscreen(self.buffer.get_insert())
+        else:
+            self.view.scroll_to_mark(self.buffer.get_insert(), 0.001, use_align=True, xalign=1.0)
 
     def scroll_to_cursor(self):
         self.view.scroll_to_mark(self.buffer.get_insert(), 0.001, use_align=True, xalign=1.0)
@@ -187,9 +255,9 @@ class Editor(SignalManager):
         from .feedback import FeedbackPopup
         return FeedbackPopup()
 
-    def message(self, message, timeout=1500):
+    def message(self, message, timeout=1500, markup=False):
         popup = self.feedback_popup
-        popup.show(self, message, timeout)
+        popup.show(self, message, timeout, markup)
         self.push_escape(popup.hide, popup.escape)
 
     def push_escape(self, callback, *args):
@@ -213,7 +281,6 @@ class Editor(SignalManager):
     def on_buffer_changed(self, buffer):
         self.last_cursor_move = None
 
-    @connect_external('view', 'button-press-event')
     def on_button_press_event(self, view, event):
         if event.button == 1:
             return True
@@ -224,4 +291,17 @@ class Editor(SignalManager):
         self.stack_add_request.emit(widget, on_popup)
 
     def popup_widget(self, widget):
-        self.stack_popup_request.emit(widget)
\ No newline at end of file
+        self.stack_popup_request.emit(widget)
+
+    def on_close(self):
+        if self.buffer.get_modified():
+            if 'MODIFIED_FILES' not in self.snaked_conf:
+                self.snaked_conf['MODIFIED_FILES'] = {}
+
+            self.snaked_conf['MODIFIED_FILES'][self.uri] = save_file(self.uri, self.utext,
+                self.encoding, True)
+        else:
+            try:
+                del self.snaked_conf['MODIFIED_FILES'][self.uri]
+            except KeyError:
+                pass
