@@ -6,24 +6,30 @@ import gtksourceview2
 import pango
 
 from ..signals import connect_all
-from ..util import (idle, lazy_property, create_lang_matchers_from_file,
-    LangGuesser, get_project_root, join_to_file_dir)
+from ..util import (idle, lazy_property, get_project_root, join_to_file_dir, join_to_settings_dir)
+
+from . import prefs
+from snaked.core import config
 
-import prefs
 from .shortcuts import register_shortcut, load_shortcuts
 from .plugins import PluginManager
 from .editor import Editor
+from .context import add_setter as add_context_setter, Processor as ContextProcessor
 
 import snaked.core.quick_open
-
+import snaked.core.titler
+import snaked.core.editor_list
 
 class EditorManager(object):
-    def __init__(self):
+    def __init__(self, session):
         self.editors = []
+        self.session = session
         self.style_manager = gtksourceview2.style_scheme_manager_get_default()
         self.lang_manager = gtksourceview2.language_manager_get_default()
         self.modify_lang_search_path(self.lang_manager)
 
+        self.on_quit = []
+
         self.plugin_manager = PluginManager()
         prefs.register_dialog('Plugins', self.plugin_manager.show_plugins_prefs, 'plugin',
             'extension')
@@ -33,27 +39,52 @@ class EditorManager(object):
         prefs.register_dialog('Editor settings', self.show_editor_preferences,
             'editor', 'font', 'style', 'margin', 'line', 'tab', 'whitespace')
 
+        prefs.register_dialog('Global preferences', self.show_global_preferences,
+            'snaked', 'global', 'preferences')
+
+        prefs.register_dialog('Session preferences', self.show_session_preferences,
+            'session', 'preferences')
+
         prefs.register_dialog('File types', self.edit_contexts, 'file', 'type', 'association')
 
+        self.snaked_conf = config.SnakedConf()
+        self.load_conf()
 
         self.escape_stack = []
         self.escape_map = {}
         self.spot_history = []
-        self.lang_gussers = {}
-
-        self.session = None
+        self.context_processors = {}
+        self.lang_contexts = {}
+        self.ctx_contexts = {}
 
         load_shortcuts()
         self.register_app_shortcuts()
 
         # Init core plugins
         self.plugin_manager.load_core_plugin(snaked.core.quick_open)
+        self.plugin_manager.load_core_plugin(snaked.core.titler)
+        self.plugin_manager.load_core_plugin(snaked.core.editor_list)
 
+        add_context_setter('lang', self.set_lang_context)
+        add_context_setter('ctx', self.set_ctx_context)
+
+    def load_conf(self):
+        self.snaked_conf.clear()
+
+        self.snaked_conf.load('snaked.conf')
+        if self.session:
+            self.snaked_conf.load(self.session + '.session')
+
+    def save_conf(self, active_editor=None):
+        self.snaked_conf['OPENED_FILES'] = [e.uri for e in self.editors if e.uri]
+        self.snaked_conf['ACTIVE_FILE'] = active_editor.uri if active_editor else None
+        self.snaked_conf.save()
 
     def register_app_shortcuts(self):
         register_shortcut('quit', '<ctrl>q', 'Application', 'Quit')
-        register_shortcut('close-window', '<ctrl>w', 'Window', 'Closes window')
-        register_shortcut('save', '<ctrl>s', 'File', 'Saves file')
+        register_shortcut('close-window', '<ctrl>w', 'Window', 'Close window')
+        register_shortcut('save', '<ctrl>s', 'File', 'Save file')
+        register_shortcut('save-all', '<ctrl><shift>s', 'File', 'Save all opened files')
         register_shortcut('new-file', '<ctrl>n', 'File',
             'Open dialog to choose new file directory and name')
         register_shortcut('show-preferences', '<ctrl>p', 'Window', 'Open preferences dialog')
@@ -65,30 +96,23 @@ class EditorManager(object):
         register_shortcut('goto-prev-spot', '<ctrl><alt>Left', 'Edit',
             'Quick jump to previous spot in history')
 
-    def get_lang_guesser(self, project_root):
-        try:
-            return self.lang_gussers[project_root]
-        except KeyError:
-            pass
-
-        guesser = None
-        if project_root:
+    def process_project_contexts(self, project_root, force=False):
+        if project_root not in self.context_processors:
             contexts_filename = os.path.join(project_root, '.snaked_project', 'contexts')
-            if os.path.exists(contexts_filename):
-                matchers = create_lang_matchers_from_file(project_root, contexts_filename)
-                guesser = LangGuesser(matchers)
+            p = self.context_processors[project_root] = ContextProcessor(project_root, contexts_filename)
+            p.process()
+        else:
+            if force:
+                self.context_processors[project_root].process()
 
-        self.lang_gussers[project_root] = guesser
-        return guesser
-
-    def open(self, filename, line=None):
-        editor = Editor()
+    def open(self, filename, line=None, contexts=None):
+        editor = Editor(self.snaked_conf)
         self.editors.append(editor)
         editor.session = self.session
 
         connect_all(self, editor)
 
-        idle(self.set_editor_prefs, editor, filename)
+        idle(self.set_editor_prefs, editor, filename, contexts)
         idle(self.set_editor_shortcuts, editor)
         idle(self.plugin_manager.editor_created, editor)
 
@@ -103,32 +127,44 @@ class EditorManager(object):
     def lang_prefs(self):
         return prefs.load_json_settings('langs.conf', {})
 
-    def set_editor_prefs(self, editor, filename):
+    def set_editor_prefs(self, editor, filename, lang_id=None):
         lang = None
         editor.lang = 'default'
-        editor.contexts = ['default']
+        editor.contexts = [editor.lang]
 
-        if filename:
-            root = get_project_root(filename)
-            guesser = self.get_lang_guesser(root)
-            if guesser:
-                contexts = guesser.guess(os.path.abspath(filename))
-                if contexts:
-                    lang_id = contexts[0]
-                    lang = self.lang_manager.get_language(lang_id)
-                    if lang:
-                        editor.lang = lang.get_id()
-                        editor.contexts = contexts
+        root = get_project_root(filename)
+        if root:
+            self.process_project_contexts(root)
 
-            if not lang:
-                lang = self.lang_manager.guess_language(filename, None)
-                if lang:
-                    editor.lang = lang.get_id()
-                    editor.contexts = [editor.lang]
+        if not lang_id and root in self.lang_contexts:
+            for id, matcher in self.lang_contexts[root].items():
+                if matcher.search(filename):
+                    lang_id = id
+                    break
+
+        if lang_id:
+            lang = self.lang_manager.get_language(lang_id)
+            if lang:
+                editor.lang = lang.get_id()
+
+        if not lang:
+            lang = self.lang_manager.guess_language(filename, None)
+            if lang:
+                editor.lang = lang.get_id()
+
+        editor.contexts = [editor.lang]
 
         if lang:
             editor.buffer.set_language(lang)
 
+        if self.session:
+            editor.contexts.append('session:' + self.session)
+
+        if root in self.ctx_contexts:
+            for ctx, matcher in self.ctx_contexts[root].items():
+                if matcher.search(filename):
+                    editor.contexts.append(ctx)
+
         pref = prefs.CompositePreferences(self.lang_prefs.get(editor.lang, {}),
             self.lang_prefs.get('default', {}), prefs.default_prefs.get(editor.lang, {}),
             prefs.default_prefs['default'])
@@ -156,11 +192,12 @@ class EditorManager(object):
 
     @Editor.editor_closed(idle=True)
     def on_editor_closed(self, editor):
+        editor.on_close()
         self.plugin_manager.editor_closed(editor)
         self.editors.remove(editor)
 
         if not self.editors:
-            snaked.core.quick_open.activate(self.get_fake_editor())
+            snaked.core.quick_open.quick_open(self.get_fake_editor())
 
     @Editor.change_title
     def on_editor_change_title(self, editor, title):
@@ -171,7 +208,7 @@ class EditorManager(object):
         self.close_editor(editor)
 
     @Editor.request_to_open_file
-    def on_request_to_open_file(self, editor, filename, line):
+    def on_request_to_open_file(self, editor, filename, line, lang_id):
         self.add_spot(editor)
 
         for e in self.editors:
@@ -183,7 +220,7 @@ class EditorManager(object):
 
                 break
         else:
-            e = self.open(filename, line)
+            e = self.open(filename, line, lang_id)
 
         return e
 
@@ -193,37 +230,34 @@ class EditorManager(object):
 
     @Editor.settings_changed(idle=True)
     def on_editor_settings_changed(self, editor):
-        self.set_editor_prefs(editor, editor.uri)
+        self.set_editor_prefs(editor, editor.uri, editor.lang)
         for e in self.editors:
             if e is not editor:
-                idle(self.set_editor_prefs, e, e.uri)
+                idle(self.set_editor_prefs, e, e.uri, e.lang)
 
     def new_file_action(self, editor):
         from snaked.core.gui import new_file
         new_file.show_create_file(editor)
 
     def quit(self, editor):
-        if self.session:
-            self.save_session(self.session, editor)
+        for e in self.editors:
+            e.on_close()
+            self.plugin_manager.editor_closed(e)
 
-        map(self.plugin_manager.editor_closed, self.editors)
+        self.save_conf(editor)
 
         self.plugin_manager.quit()
 
+        for q in self.on_quit:
+            try:
+                q()
+            except:
+                import traceback
+                traceback.print_exc()
+
         if gtk.main_level() > 0:
             gtk.main_quit()
 
-    def get_session_settings(self, session):
-        from .prefs import load_json_settings
-        return load_json_settings('%s.session' % session, {})
-
-    def save_session(self, session, active_editor=None):
-        from .prefs import save_json_settings
-        settings = self.get_session_settings(session)
-        settings['files'] = [e.uri for e in self.editors if e.uri]
-        settings['active_file'] = active_editor.uri if active_editor else None
-        save_json_settings('%s.session' % session, settings)
-
     @Editor.push_escape_callback
     def on_push_escape_callback(self, editor, callback, args):
         key = (callback,) + tuple(map(weakref.ref, args))
@@ -323,6 +357,19 @@ class EditorManager(object):
 
         self.goto_last_spot(editor)
 
+    def show_global_preferences(self, editor):
+        self.save_conf(editor)
+        e = editor.open_file(join_to_settings_dir('snaked.conf'), lang_id='python')
+        e.file_saved.connect(self, 'on_config_saved')
+
+    def show_session_preferences(self, editor):
+        self.save_conf(editor)
+        e = editor.open_file(join_to_settings_dir(self.session + '.session'), lang_id='python')
+        e.file_saved.connect(self, 'on_config_saved')
+
+    def on_config_saved(self, editor):
+        editor.message('Config updated')
+        self.load_conf()
 
     def edit_contexts(self, editor):
         import shutil
@@ -339,7 +386,7 @@ class EditorManager(object):
 
     def on_context_saved(self, editor):
         editor.message('File type associations changed')
-        self.lang_gussers.clear()
+        self.process_project_contexts(editor.project_root, True)
 
     def modify_lang_search_path(self, manager):
         search_path = manager.get_search_path()
@@ -351,6 +398,16 @@ class EditorManager(object):
         search_path.insert(i, join_to_file_dir(__file__, 'lang-specs'))
         manager.set_search_path(search_path)
 
+    def set_lang_context(self, project_root, contexts):
+        self.lang_contexts[project_root] = contexts
+
+    def set_ctx_context(self, project_root, contexts):
+        self.ctx_contexts[project_root] = contexts
+
+    def save_all(self, editor):
+        for e in self.editors:
+            e.save()
+
 class EditorSpot(object):
     def __init__(self, manager, editor):
         self.manager = manager
@@ -388,6 +445,7 @@ class FakeEditor(object):
         self.manager = manager
         self.request_transient_for = self
         self.session = manager.session
+        self.snaked_conf = manager.snaked_conf
 
     def emit(self, window):
         self.manager.set_transient_for(self, window)
@@ -403,3 +461,6 @@ class FakeEditor(object):
 
     def message(self, message, timeout=None):
         print message
+
+    def get_project_root(*args, **kwargs):
+        return None
