@@ -1,4 +1,4 @@
-author = 'Anton Bobrov<bobrov@vl.ru>'
+eauthor = 'Anton Bobrov<bobrov@vl.ru>'
 name = 'Edit and Select'
 desc = 'Various edit shortcuts'
 
@@ -16,12 +16,22 @@ def init(manager):
         'Show cursor offset and column', show_offset)
     manager.add_shortcut('wrap-text', '<alt>f', 'Edit', 'Wrap text on right margin width', wrap_text)
 
+    manager.add_global_option('DOUBLE_BRACKET_MATCHER', True, "Enable custom bracket matcher")
+    manager.add_global_option('COPY_DELETED_LINE', True, "Put deleted line into clipboard")
+
+def editor_created(editor):
+    if editor.snaked_conf['DOUBLE_BRACKET_MATCHER']:
+        from bracket_matcher import attach
+        attach(editor)
+
 def delete_line(editor):
-    from util import get_line_bounds
+    from util import get_line_bounds, line_is_empty
+
     bounds = get_line_bounds(editor.cursor)
-    clipboard = editor.view.get_clipboard(gtk.gdk.SELECTION_CLIPBOARD)
-    editor.buffer.select_range(*bounds)
-    editor.buffer.copy_clipboard(clipboard)
+    if not line_is_empty(editor.cursor) and editor.snaked_conf['COPY_DELETED_LINE']:
+        clipboard = editor.view.get_clipboard(gtk.gdk.SELECTION_CLIPBOARD)
+        editor.buffer.select_range(*bounds)
+        editor.buffer.copy_clipboard(clipboard)
 
     editor.buffer.begin_user_action()
     editor.buffer.delete(*bounds)
