@@ -28,12 +28,19 @@ def init(manager):
     manager.add_shortcut('toggle-test-panel', '<alt>1', 'Window',
         'Toggle test panel', toggle_test_panel)
 
+    manager.add_global_option('PYTHON_SPYPKG_HANDLER_MAX_CHARS', 25,
+        'Maximum allowed python package title length')
+
+    manager.add_title_handler('pypkg', pypkg_handler)
+    manager.add_title_handler('spypkg', spypkg_handler)
+
     from snaked.core.prefs import register_dialog
     register_dialog('Rope hints', edit_rope_hints, 'rope', 'hints')
     register_dialog('Rope config', edit_rope_config, 'rope', 'config')
 
+
 def editor_created(editor):
-    editor.connect('get-title', on_editor_get_title)
+    editor.connect('get-project-larva', on_editor_get_project_larva)
 
 def editor_opened(editor):
     from plugin import Plugin
@@ -41,7 +48,10 @@ def editor_opened(editor):
     handlers[editor] = h
 
 def editor_closed(editor):
-    del handlers[editor]
+    try:
+        del handlers[editor]
+    except KeyError:
+        pass
 
 def quit():
     global outline_dialog
@@ -69,15 +79,39 @@ def show_calltips(editor):
 
     h.show_calltips()
 
-def on_editor_get_title(editor):
-    if editor.uri.endswith('.py'):
-        return get_python_title(editor.uri)
+def get_package_root(module_path):
+    import os.path
 
-    return None
+    packages = [os.path.basename(module_path).rpartition('.')[0]]
+    while True:
+        path = os.path.dirname(module_path)
+        if path == module_path:
+            break
+
+        module_path = path
+
+        if os.path.exists(os.path.join(module_path, '__init__.py')):
+            packages.append(os.path.basename(module_path))
+        else:
+            break
+
+    return module_path, '.'.join(reversed(packages))
+
+def on_editor_get_project_larva(editor):
+    from snaked.util import join_to_file_dir
+    import os.path
+
+    if os.path.exists(join_to_file_dir(editor.uri, '__init__.py')):
+        editor.get_project_larva.stop_emission()
+        root, packages = get_package_root(editor.uri)
+        return os.path.join(root, packages.partition('.')[0])
+
+def pypkg_handler(editor):
+    uri = editor.uri
+    if not uri.endswith('.py'):
+        return None
 
-def get_python_title(uri):
     from os.path import dirname, basename, exists, join
-
     title = basename(uri)
     packages = []
     while True:
@@ -100,6 +134,31 @@ def get_python_title(uri):
     else:
         return None
 
+def spypkg_handler(editor):
+    package = pypkg_handler(editor)
+    if not package:
+        return None
+
+    import re
+
+    max_chars_in_title = editor.snaked_conf['PYTHON_SPYPKG_HANDLER_MAX_CHARS']
+
+    parts = package.split('.')
+    if parts < 3 or len(package) <= max_chars_in_title:
+        return package
+
+    index = len(parts) - 2
+    while len(package) > max_chars_in_title:
+        parts[index] = '.'
+        index -= 1
+        package = '.'.join(parts)
+        package = re.sub(r'\.{4,}', '...', package)
+
+        if index == 0:
+            break
+
+    return package
+
 def open_outline(editor):
     global outline_dialog
     if not outline_dialog:
@@ -113,6 +172,12 @@ def edit_rope_hints(editor):
     from os.path import join, exists, dirname
     from snaked.util import make_missing_dirs
 
+    if not editor.project_root:
+        editor.message('Can not determine current project.\n'
+            'Are you editing read-only file?\n'
+            'Also check existence of .snaked_project directory', 8000)
+        return
+
     ropehints = join(editor.project_root, '.ropeproject', 'ropehints.py')
     if not exists(ropehints):
         make_missing_dirs(ropehints)
@@ -122,14 +187,27 @@ def edit_rope_hints(editor):
 
 def edit_rope_config(editor):
     from os.path import join, exists
+
+    if not editor.project_root:
+        editor.message('Can not determine current project.\n'
+            'Are you editing read-only file?\n'
+            'Also check existence of .snaked_project directory', 8000)
+        return
+
     ropeconfig = join(editor.project_root, '.ropeproject', 'config.py')
-    handlers[editor].project_manager
 
     if exists(ropeconfig):
         editor.open_file(ropeconfig)
     else:
-        editor.message('There is no existing rope config.\n'
-            'Are you sure this is python project?', 5000)
+        if editor in handlers:
+            handlers[editor].project_manager
+
+        if exists(ropeconfig):
+            editor.open_file(ropeconfig)
+        else:
+            editor.message('There is no existing rope config.\n'
+                'Are you sure this is python project?\n'
+                'Try to open any python file', 8000)
 
 def get_pytest_runner(editor):
     """:rtype: snaked.plugins.python.pytest_runner.TestRunner()"""
