@@ -13,6 +13,10 @@ def pangonify_rst(text):
     result = ''
 
     lines = text.strip().expandtabs().splitlines()
+
+    if len(lines) > 1 and lines[1].strip():
+        lines = [lines[0]] + [''] + lines[1:]
+
     indent = 1000
     for l in lines[1:]:
         stripped = l.lstrip()
@@ -110,7 +114,6 @@ class RopeCompletionProvider(gobject.GObject, CompletionProvider):
 
     def do_populate(self, context):
         project = self.plugin().project_manager.project
-
         from rope.contrib import codeassist
 
         try:
