@@ -1,7 +1,6 @@
 import os
 import shutil
 from os.path import join, dirname, realpath, abspath, exists, expanduser
-import re
 
 import gobject
 import gtk
@@ -15,9 +14,13 @@ def idle_callback(callable, args):
     return False
 
 def idle(callable, *args, **kwargs):
-    return gobject.idle_add(idle_callback, callable, (args, kwargs))
+    options = {}
+    if 'priority' in kwargs:
+        options['priority'] = kwargs['priority']
+        del kwargs['priority']
+    return gobject.idle_add(idle_callback, callable, (args, kwargs), **options)
 
-def save_file(filename, data, encoding):
+def save_file(filename, data, encoding, keep_tmp=False):
     tmpfilename = realpath(filename) + '.bak'
 
     try:
@@ -34,9 +37,15 @@ def save_file(filename, data, encoding):
     f.close()
 
     if exists(filename):
-        shutil.copymode(filename, tmpfilename)
+        try:
+            shutil.copymode(filename, tmpfilename)
+        except OSError:
+            pass
 
-    os.rename(tmpfilename, filename)
+    if keep_tmp:
+        return tmpfilename
+    else:
+        os.rename(tmpfilename, filename)
 
 def connect(sender, signal, obj, attr, idle=False, after=False):
     return Handler(weak_connect(
@@ -49,6 +58,10 @@ def join_to_settings_dir(*args):
     config_dir = os.getenv('XDG_CONFIG_HOME', expanduser('~/.config'))
     return join(config_dir, 'snaked', *args)
 
+def join_to_cache_dir(*args):
+    config_dir = os.getenv('XDG_CACHE_HOME', expanduser('~/.cache'))
+    return join(config_dir, 'snaked', *args)
+
 def make_missing_dirs(filename):
     path = dirname(filename)
     if not exists(path):
@@ -121,29 +134,27 @@ def set_activate_the_one_item(entry, treeview):
 
     entry.connect('activate', activate)
 
-def create_lang_matchers_from_file(project_root, filename):
-    result = []
-    for l in open(filename):
-        l = l.strip()
-        if not l or l.startswith('#'):
-            continue
+def mimic_to_sourceview_theme(textview, sourceview):
+    """Gets font and bg settings from gtksourceview and applies it to textview
 
-        try:
-            expr, contexts = l.split(':')
-        except ValueError:
-            continue
+    :type textview: gtk.TextView()
+    :type sourceview: gtksourceview2.View()
 
-        contexts = [c.strip() for c in contexts.split(',')]
-        if not contexts:
-            continue
+    """
 
-        if not l.startswith('/'):
-            expr = '/*/' + expr
+    style = sourceview.get_buffer().get_style_scheme().get_style('text')
+    if style:
+        if style.props.background_set:
+            textview.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse(style.props.background))
+        if style.props.foreground_set:
+            textview.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse(style.props.foreground))
 
-        matcher = re.compile(project_root + re.escape(expr).replace('\*', '.*') + '$')
-        result.append((matcher, contexts))
-
-    return result
+    style = sourceview.get_buffer().get_style_scheme().get_style('selection')
+    if style:
+        if style.props.background_set:
+            textview.modify_base(gtk.STATE_SELECTED, gtk.gdk.color_parse(style.props.background))
+        if style.props.foreground_set:
+            textview.modify_text(gtk.STATE_SELECTED, gtk.gdk.color_parse(style.props.foreground))
 
 
 class BuilderAware(object):
@@ -158,16 +169,4 @@ class BuilderAware(object):
             raise AttributeError('Builder have no %s object' % name)
 
         setattr(self, name, obj)
-        return obj
-
-
-class LangGuesser(object):
-    def __init__(self, matchers):
-        self.matchers = matchers
-
-    def guess(self, filename):
-        for m, ctx in self.matchers:
-            if m.match(filename):
-                return ctx
-
-        return []
\ No newline at end of file
+        return obj
\ No newline at end of file
